@@ -1,5 +1,6 @@
-/* example-start eventbox eventbox.c */
 
+#include <config.h>
+#include <stdlib.h>
 #include <gtk/gtk.h>
 
 int main( int argc,
@@ -15,15 +16,15 @@ int main( int argc,
     
     gtk_window_set_title (GTK_WINDOW (window), "Event Box");
     
-    gtk_signal_connect (GTK_OBJECT (window), "destroy",
-			GTK_SIGNAL_FUNC (gtk_exit), NULL);
+    g_signal_connect (G_OBJECT (window), "destroy",
+	              G_CALLBACK (exit), NULL);
     
     gtk_container_set_border_width (GTK_CONTAINER (window), 10);
     
     /* Create an EventBox and add it to our toplevel window */
     
     event_box = gtk_event_box_new ();
-    gtk_container_add (GTK_CONTAINER(window), event_box);
+    gtk_container_add (GTK_CONTAINER (window), event_box);
     gtk_widget_show (event_box);
     
     /* Create a long label */
@@ -33,12 +34,12 @@ int main( int argc,
     gtk_widget_show (label);
     
     /* Clip it short. */
-    gtk_widget_set_usize (label, 110, 20);
+    gtk_widget_set_size_request (label, 110, 20);
     
     /* And bind an action to it */
     gtk_widget_set_events (event_box, GDK_BUTTON_PRESS_MASK);
-    gtk_signal_connect (GTK_OBJECT(event_box), "button_press_event",
-			GTK_SIGNAL_FUNC (gtk_exit), NULL);
+    g_signal_connect (G_OBJECT (event_box), "button_press_event",
+	              G_CALLBACK (exit), NULL);
     
     /* Yet one more thing you need an X window for ... */
     
@@ -49,6 +50,5 @@ int main( int argc,
     
     gtk_main ();
     
-    return(0);
+    return 0;
 }
-/* example-end */
