@@ -24,6 +24,7 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
  */
 
+#include <config.h>
 #include <string.h>		/* For memset() */
 
 #include "gdk.h"
@@ -46,10 +47,6 @@ GdkEventFunc   _gdk_event_func = NULL;    /* Callback for events */
 gpointer       _gdk_event_data = NULL;
 GDestroyNotify _gdk_event_notify = NULL;
 
-#define TRIPLE_CLICK_TIME(display) (2*display->double_click_time)
-#define DOUBLE_CLICK_DIST      5
-#define TRIPLE_CLICK_DIST      5
-
 /*********************************************
  * Functions for maintaining the event queue *
  *********************************************/
@@ -268,6 +265,8 @@ static GHashTable *event_hash = NULL;
  * 
  * Return value: a newly-allocated #GdkEvent. The returned #GdkEvent 
  * should be freed with gdk_event_free().
+ *
+ * Since: 2.2
  **/
 GdkEvent*
 gdk_event_new (GdkEventType type)
@@ -361,7 +360,6 @@ gdk_event_is_allocated (GdkEvent *event)
 GdkEvent*
 gdk_event_copy (GdkEvent *event)
 {
-  GdkEventPrivate *private;
   GdkEventPrivate *new_private;
   GdkEvent *new_event;
   
@@ -370,13 +368,16 @@ gdk_event_copy (GdkEvent *event)
   new_event = gdk_event_new (GDK_NOTHING);
   new_private = (GdkEventPrivate *)new_event;
 
-  private = (GdkEventPrivate *)event;
-
   *new_event = *event;
   if (new_event->any.window)
     g_object_ref (new_event->any.window);
 
-  new_private->screen = private->screen;
+  if (gdk_event_is_allocated (event))
+    {
+      GdkEventPrivate *private = (GdkEventPrivate *)event;
+
+      new_private->screen = private->screen;
+    }
   
   switch (event->any.type)
     {
@@ -408,6 +409,20 @@ gdk_event_copy (GdkEvent *event)
     case GDK_SETTING:
       new_event->setting.name = g_strdup (new_event->setting.name);
       break;
+
+    case GDK_BUTTON_PRESS:
+    case GDK_BUTTON_RELEASE:
+      if (event->button.axes) 
+	new_event->button.axes = g_memdup (event->button.axes, 
+					     sizeof (gdouble) * event->button.device->num_axes);
+      break;
+
+    case GDK_MOTION_NOTIFY:
+      if (event->motion.axes) 
+	new_event->motion.axes = g_memdup (event->motion.axes, 
+					   sizeof (gdouble) * event->motion.device->num_axes);
+      
+      break;
       
     default:
       break;
@@ -828,6 +843,8 @@ gdk_event_get_axis (GdkEvent   *event,
  * Sets the screen for @event to @screen. The event must
  * have been allocated by GTK+, for instance, by
  * gdk_event_copy().
+ *
+ * Since: 2.2
  **/
 void
 gdk_event_set_screen (GdkEvent  *event,
@@ -847,14 +864,16 @@ gdk_event_set_screen (GdkEvent  *event,
  * @event: a #GdkEvent
  * 
  * Returns the screen for the event. The screen is
- * typically the screen for event->any.window, but
+ * typically the screen for <literal>event->any.window</literal>, but
  * for events such as mouse events, it is the screen
  * where the the pointer was when the event occurs -
  * that is, the screen which has the root window 
- * to which event->motion.x_root and
- * event->motion.y_root are relative.
+ * to which <literal>event->motion.x_root</literal> and
+ * <literal>event->motion.y_root</literal> are relative.
  * 
  * Return value: the screen for the event
+ *
+ * Since: 2.2
  **/
 GdkScreen *
 gdk_event_get_screen (GdkEvent *event)
@@ -1009,22 +1028,28 @@ void
 _gdk_event_button_generate (GdkDisplay *display,
 			    GdkEvent   *event)
 {
-  if ((event->button.time < (display->button_click_time[1] + TRIPLE_CLICK_TIME (display))) &&
+  if ((event->button.time < (display->button_click_time[1] + 2*display->double_click_time)) &&
       (event->button.window == display->button_window[1]) &&
-      (event->button.button == display->button_number[1]))
-    {
+      (event->button.button == display->button_number[1]) &&
+      (ABS (event->button.x - display->button_x[1]) <= display->double_click_distance) &&
+      (ABS (event->button.y - display->button_y[1]) <= display->double_click_distance))
+{
       gdk_synthesize_click (display, event, 3);
-      
+            
       display->button_click_time[1] = 0;
       display->button_click_time[0] = 0;
       display->button_window[1] = NULL;
       display->button_window[0] = 0;
       display->button_number[1] = -1;
       display->button_number[0] = -1;
+      display->button_x[0] = display->button_x[1] = 0;
+      display->button_y[0] = display->button_y[1] = 0;
     }
   else if ((event->button.time < (display->button_click_time[0] + display->double_click_time)) &&
 	   (event->button.window == display->button_window[0]) &&
-	   (event->button.button == display->button_number[0]))
+	   (event->button.button == display->button_number[0]) &&
+	   (ABS (event->button.x - display->button_x[0]) <= display->double_click_distance) &&
+	   (ABS (event->button.y - display->button_y[0]) <= display->double_click_distance))
     {
       gdk_synthesize_click (display, event, 2);
       
@@ -1034,6 +1059,10 @@ _gdk_event_button_generate (GdkDisplay *display,
       display->button_window[0] = event->button.window;
       display->button_number[1] = display->button_number[0];
       display->button_number[0] = event->button.button;
+      display->button_x[1] = display->button_x[0];
+      display->button_x[0] = event->button.x;
+      display->button_y[1] = display->button_y[0];
+      display->button_y[0] = event->button.y;
     }
   else
     {
@@ -1043,6 +1072,10 @@ _gdk_event_button_generate (GdkDisplay *display,
       display->button_window[0] = event->button.window;
       display->button_number[1] = -1;
       display->button_number[0] = event->button.button;
+      display->button_x[1] = 0;
+      display->button_x[0] = event->button.x;
+      display->button_y[1] = 0;
+      display->button_y[0] = event->button.y;
     }
 }
 
@@ -1104,7 +1137,10 @@ gdk_synthesize_window_state (GdkWindow     *window,
  * 
  * Sets the double click time (two clicks within this time interval
  * count as a double click and result in a #GDK_2BUTTON_PRESS event).
- * Applications should NOT set this, it is a global user-configured setting.
+ * Applications should <emphasis>not</emphasis> set this, it is a global 
+ * user-configured setting.
+ *
+ * Since: 2.2
  **/
 void
 gdk_display_set_double_click_time (GdkDisplay *display,
@@ -1118,8 +1154,10 @@ gdk_display_set_double_click_time (GdkDisplay *display,
  * @msec: double click time in milliseconds (thousandths of a second)
  *
  * Set the double click time for the default display. See
- * gdk_display_set_double_click_time(). Applications should NOT
- * set this, it is a global user-configured setting.
+ * gdk_display_set_double_click_time(). 
+ * See also gdk_display_set_double_click_distance().
+ * Applications should <emphasis>not</emphasis> set this, it is a 
+ * global user-configured setting.
  **/
 void
 gdk_set_double_click_time (guint msec)
@@ -1127,6 +1165,26 @@ gdk_set_double_click_time (guint msec)
   gdk_display_set_double_click_time (gdk_display_get_default (), msec);
 }
 
+/**
+ * gdk_display_set_double_click_distance:
+ * @display: a #GdkDisplay
+ * @distance: distance in pixels
+ * 
+ * Sets the double click distance (two clicks within this distance
+ * count as a double click and result in a #GDK_2BUTTON_PRESS event).
+ * See also gdk_display_set_double_click_time().
+ * Applications should <emphasis>not</emphasis> set this, it is a global 
+ * user-configured setting.
+ *
+ * Since: 2.4
+ **/
+void
+gdk_display_set_double_click_distance (GdkDisplay *display,
+				       guint       distance)
+{
+  display->double_click_distance = distance;
+}
+
 GType
 gdk_event_get_type (void)
 {
