@@ -20,6 +20,7 @@
  *
  * Author:  Owen Taylor, Red Hat, Inc.
  */
+#include <config.h>
 #include <limits.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -383,9 +384,13 @@ read_settings (XSettingsClient *client)
 				   &type, &format, &n_items, &bytes_after, &data);
       XSetErrorHandler (old_handler);
       
-      if (result == Success && type == client->xsettings_atom)
+      if (result == Success && type != None)
 	{
-	  if (format != 8)
+	  if (type != client->xsettings_atom)
+	    {
+	      fprintf (stderr, "Invalid type for XSETTINGS property");
+	    }
+	  else if (format != 8)
 	    {
 	      fprintf (stderr, "Invalid format for XSETTINGS property %d", format);
 	    }
