@@ -30,9 +30,11 @@
 #define __GTK_CALENDAR_H__
 
 #include <gdk/gdk.h>
-#include <gtk/gtksignal.h>
 #include <gtk/gtkwidget.h>
 
+/* Not needed, retained for compatibility -Yosh */
+#include <gtk/gtksignal.h>
+
 
 #ifdef __cplusplus
 extern "C" {
@@ -40,11 +42,11 @@ extern "C" {
 
 
 #define GTK_TYPE_CALENDAR                  (gtk_calendar_get_type ())
-#define GTK_CALENDAR(obj)                  (GTK_CHECK_CAST ((obj), GTK_TYPE_CALENDAR, GtkCalendar))
-#define GTK_CALENDAR_CLASS(klass)          (GTK_CHECK_CLASS_CAST ((klass), GTK_TYPE_CALENDAR, GtkCalendarClass))
-#define GTK_IS_CALENDAR(obj)               (GTK_CHECK_TYPE ((obj), GTK_TYPE_CALENDAR))
-#define GTK_IS_CALENDAR_CLASS(klass)       (GTK_CHECK_CLASS_TYPE ((klass), GTK_TYPE_CALENDAR))
-#define GTK_CALENDAR_GET_CLASS(obj)        (GTK_CHECK_GET_CLASS ((obj), GTK_TYPE_CALENDAR, GtkCalendarClass))
+#define GTK_CALENDAR(obj)                  (G_TYPE_CHECK_INSTANCE_CAST ((obj), GTK_TYPE_CALENDAR, GtkCalendar))
+#define GTK_CALENDAR_CLASS(klass)          (G_TYPE_CHECK_CLASS_CAST ((klass), GTK_TYPE_CALENDAR, GtkCalendarClass))
+#define GTK_IS_CALENDAR(obj)               (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GTK_TYPE_CALENDAR))
+#define GTK_IS_CALENDAR_CLASS(klass)       (G_TYPE_CHECK_CLASS_TYPE ((klass), GTK_TYPE_CALENDAR))
+#define GTK_CALENDAR_GET_CLASS(obj)        (G_TYPE_INSTANCE_GET_CLASS ((obj), GTK_TYPE_CALENDAR, GtkCalendarClass))
 
 
 typedef struct _GtkCalendar	       GtkCalendar;
@@ -89,6 +91,12 @@ struct _GtkCalendar
   
   gpointer private_data;
   gchar grow_space [32];
+
+  /* Padding for future expansion */
+  void (*_gtk_reserved1) (void);
+  void (*_gtk_reserved2) (void);
+  void (*_gtk_reserved3) (void);
+  void (*_gtk_reserved4) (void);
 };
 
 struct _GtkCalendarClass
@@ -107,7 +115,7 @@ struct _GtkCalendarClass
 };
 
 
-GtkType	   gtk_calendar_get_type	(void) G_GNUC_CONST;
+GType	   gtk_calendar_get_type	(void) G_GNUC_CONST;
 GtkWidget* gtk_calendar_new		(void);
 
 gboolean   gtk_calendar_select_month	(GtkCalendar *calendar, 
@@ -123,8 +131,14 @@ gboolean   gtk_calendar_unmark_day	(GtkCalendar *calendar,
 void	   gtk_calendar_clear_marks	(GtkCalendar *calendar);
 
 
+void	   gtk_calendar_set_display_options (GtkCalendar    	      *calendar,
+					     GtkCalendarDisplayOptions flags);
+GtkCalendarDisplayOptions
+           gtk_calendar_get_display_options (GtkCalendar   	      *calendar);
+#ifndef GTK_DISABLE_DEPRECATED
 void	   gtk_calendar_display_options (GtkCalendar		  *calendar,
 					 GtkCalendarDisplayOptions flags);
+#endif
 
 void	   gtk_calendar_get_date	(GtkCalendar *calendar, 
 					 guint	     *year,
