@@ -17,6 +17,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#include <config.h>
 #include "gtkcellrenderer.h"
 #include "gtkintl.h"
 #include "gtkmarshalers.h"
@@ -125,6 +126,7 @@ gtk_cell_renderer_class_init (GtkCellRendererClass *class)
 
   /**
    * GtkCellRenderer::editing-canceled:
+   * @renderer: the object which received the signal
    *
    * This signal gets emitted when the user cancels the process of editing a
    * cell.  For example, an editable cell renderer could be written to cancel
@@ -191,7 +193,7 @@ gtk_cell_renderer_class_init (GtkCellRendererClass *class)
 						      P_("xpad"),
 						      P_("The xpad"),
 						      0,
-						      100,
+						      G_MAXUINT,
 						      2,
 						      G_PARAM_READABLE |
 						      G_PARAM_WRITABLE));
@@ -202,7 +204,7 @@ gtk_cell_renderer_class_init (GtkCellRendererClass *class)
 						      P_("ypad"),
 						      P_("The ypad"),
 						      0,
-						      100,
+						      G_MAXUINT,
 						      2,
 						      G_PARAM_READABLE |
 						      G_PARAM_WRITABLE));
@@ -213,7 +215,7 @@ gtk_cell_renderer_class_init (GtkCellRendererClass *class)
 						     P_("width"),
 						     P_("The fixed width"),
 						     -1,
-						     100,
+						     G_MAXINT,
 						     -1,
 						     G_PARAM_READABLE |
 						     G_PARAM_WRITABLE));
@@ -224,7 +226,7 @@ gtk_cell_renderer_class_init (GtkCellRendererClass *class)
 						     P_("height"),
 						     P_("The fixed height"),
 						     -1,
-						     100,
+						     G_MAXINT,
 						     -1,
 						     G_PARAM_READABLE |
 						     G_PARAM_WRITABLE));
