@@ -982,6 +982,8 @@ gtk_cell_renderer_text_set_property (GObject      *object,
           font_desc = pango_font_description_from_string (name);
 
         set_font_description (celltext, font_desc);
+
+	pango_font_description_free (font_desc);
         
 	if (celltext->fixed_height_rows != -1)
 	  celltext->calc_fixed_height = TRUE;
@@ -1380,6 +1382,11 @@ gtk_cell_renderer_text_render (GtkCellRenderer      *cell,
       else
 	state = GTK_STATE_ACTIVE;
     }
+  else if ((flags & GTK_CELL_RENDERER_PRELIT) == GTK_CELL_RENDERER_PRELIT &&
+	   GTK_WIDGET_STATE (widget) == GTK_STATE_PRELIGHT)
+    {
+      state = GTK_STATE_PRELIGHT;
+    }
   else
     {
       if (GTK_WIDGET_STATE (widget) == GTK_STATE_INSENSITIVE)
