@@ -24,18 +24,31 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
  */
 
+#include <config.h>
 #include "gtkbutton.h"
 #include "gtkdialog.h"
 #include "gtkhbbox.h"
 #include "gtkhseparator.h"
 #include "gtkmarshalers.h"
 #include "gtkvbox.h"
-#include "gtksignal.h"
 #include "gdkkeysyms.h"
 #include "gtkmain.h"
 #include "gtkintl.h"
 #include "gtkbindings.h"
 
+#define GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GTK_TYPE_DIALOG, GtkDialogPrivate))
+
+typedef struct {
+  guint ignore_separator : 1;
+} GtkDialogPrivate;
+
+typedef struct _ResponseData ResponseData;
+
+struct _ResponseData
+{
+  gint response_id;
+};
+
 static void gtk_dialog_class_init (GtkDialogClass *klass);
 static void gtk_dialog_init       (GtkDialog      *dialog);
 
@@ -61,6 +74,9 @@ static void gtk_dialog_map               (GtkWidget        *widget);
 
 static void gtk_dialog_close             (GtkDialog        *dialog);
 
+static ResponseData* get_response_data   (GtkWidget        *widget,
+					  gboolean          create);
+
 enum {
   PROP_0,
   PROP_HAS_SEPARATOR
@@ -75,26 +91,28 @@ enum {
 static gpointer parent_class;
 static guint dialog_signals[LAST_SIGNAL];
 
-GtkType
+GType
 gtk_dialog_get_type (void)
 {
-  static GtkType dialog_type = 0;
+  static GType dialog_type = 0;
 
   if (!dialog_type)
     {
-      static const GtkTypeInfo dialog_info =
+      static const GTypeInfo dialog_info =
       {
-	"GtkDialog",
-	sizeof (GtkDialog),
 	sizeof (GtkDialogClass),
-	(GtkClassInitFunc) gtk_dialog_class_init,
-	(GtkObjectInitFunc) gtk_dialog_init,
-	/* reserved_1 */ NULL,
-        /* reserved_2 */ NULL,
-        (GtkClassInitFunc) NULL,
+	NULL,		/* base_init */
+	NULL,		/* base_finalize */
+	(GClassInitFunc) gtk_dialog_class_init,
+	NULL,		/* class_finalize */
+	NULL,		/* class_data */
+	sizeof (GtkDialog),
+	0,		/* n_preallocs */
+	(GInstanceInitFunc) gtk_dialog_init,
       };
 
-      dialog_type = gtk_type_unique (GTK_TYPE_WINDOW, &dialog_info);
+      dialog_type = g_type_register_static (GTK_TYPE_WINDOW, "GtkDialog",
+					    &dialog_info, 0);
     }
 
   return dialog_type;
@@ -104,12 +122,10 @@ static void
 gtk_dialog_class_init (GtkDialogClass *class)
 {
   GObjectClass *gobject_class;
-  GtkObjectClass *object_class;
   GtkWidgetClass *widget_class;
   GtkBindingSet *binding_set;
   
   gobject_class = G_OBJECT_CLASS (class);
-  object_class = GTK_OBJECT_CLASS (class);
   widget_class = GTK_WIDGET_CLASS (class);
   
   parent_class = g_type_class_peek_parent (class);
@@ -122,43 +138,47 @@ gtk_dialog_class_init (GtkDialogClass *class)
 
   class->close = gtk_dialog_close;
   
+  g_type_class_add_private (gobject_class, sizeof (GtkDialogPrivate));
+
   g_object_class_install_property (gobject_class,
                                    PROP_HAS_SEPARATOR,
                                    g_param_spec_boolean ("has_separator",
-							 _("Has separator"),
-							 _("The dialog has a separator bar above its buttons"),
+							 P_("Has separator"),
+							 P_("The dialog has a separator bar above its buttons"),
                                                          TRUE,
                                                          G_PARAM_READWRITE));
   
   dialog_signals[RESPONSE] =
-    gtk_signal_new ("response",
-                    GTK_RUN_LAST,
-                    GTK_CLASS_TYPE (object_class),
-                    GTK_SIGNAL_OFFSET (GtkDialogClass, response),
-                    _gtk_marshal_NONE__INT,
-		    GTK_TYPE_NONE, 1,
-                    GTK_TYPE_INT);
+    g_signal_new ("response",
+		  G_OBJECT_CLASS_TYPE (class),
+		  G_SIGNAL_RUN_LAST,
+		  G_STRUCT_OFFSET (GtkDialogClass, response),
+		  NULL, NULL,
+		  _gtk_marshal_NONE__INT,
+		  G_TYPE_NONE, 1,
+		  G_TYPE_INT);
 
   dialog_signals[CLOSE] =
-    gtk_signal_new ("close",
-                    GTK_RUN_LAST | GTK_RUN_ACTION,
-                    GTK_CLASS_TYPE (object_class),
-                    GTK_SIGNAL_OFFSET (GtkDialogClass, close),
-                    _gtk_marshal_NONE__NONE,
-		    GTK_TYPE_NONE, 0);
+    g_signal_new ("close",
+		  G_OBJECT_CLASS_TYPE (class),
+		  G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
+		  G_STRUCT_OFFSET (GtkDialogClass, close),
+		  NULL, NULL,
+		  _gtk_marshal_NONE__NONE,
+		  G_TYPE_NONE, 0);
   
   gtk_widget_class_install_style_property (widget_class,
 					   g_param_spec_int ("content_area_border",
-                                                             _("Content area border"),
-                                                             _("Width of border around the main dialog area"),
+                                                             P_("Content area border"),
+                                                             P_("Width of border around the main dialog area"),
                                                              0,
                                                              G_MAXINT,
                                                              2,
                                                              G_PARAM_READABLE));
   gtk_widget_class_install_style_property (widget_class,
                                            g_param_spec_int ("button_spacing",
-                                                             _("Button spacing"),
-                                                             _("Spacing between buttons"),
+                                                             P_("Button spacing"),
+                                                             P_("Spacing between buttons"),
                                                              0,
                                                              G_MAXINT,
                                                              10,
@@ -166,8 +186,8 @@ gtk_dialog_class_init (GtkDialogClass *class)
   
   gtk_widget_class_install_style_property (widget_class,
                                            g_param_spec_int ("action_area_border",
-                                                             _("Action area border"),
-                                                             _("Width of border around the button area at the bottom of the dialog"),
+                                                             P_("Action area border"),
+                                                             P_("Width of border around the button area at the bottom of the dialog"),
                                                              0,
                                                              G_MAXINT,
                                                              5,
@@ -209,14 +229,19 @@ update_spacings (GtkDialog *dialog)
 static void
 gtk_dialog_init (GtkDialog *dialog)
 {
+  GtkDialogPrivate *priv;
+
+  priv = GET_PRIVATE (dialog);
+  priv->ignore_separator = FALSE;
+
   /* To avoid breaking old code that prevents destroy on delete event
    * by connecting a handler, we have to have the FIRST signal
    * connection on the dialog.
    */
-  gtk_signal_connect (GTK_OBJECT (dialog),
-                      "delete_event",
-                      GTK_SIGNAL_FUNC (gtk_dialog_delete_event_handler),
-                      NULL);
+  g_signal_connect (dialog,
+                    "delete_event",
+                    G_CALLBACK (gtk_dialog_delete_event_handler),
+                    NULL);
   
   dialog->vbox = gtk_vbox_new (FALSE, 0);
   
@@ -344,31 +369,54 @@ gtk_dialog_style_set (GtkWidget *widget,
   update_spacings (GTK_DIALOG (widget));
 }
 
+static gboolean
+dialog_has_cancel (GtkDialog *dialog)
+{
+  GList *children, *tmp_list;
+  gboolean ret = FALSE;
+      
+  children = gtk_container_get_children (GTK_CONTAINER (dialog->action_area));
+
+  for (tmp_list = children; tmp_list; tmp_list = tmp_list->next)
+    {
+      ResponseData *rd = get_response_data (tmp_list->data, FALSE);
+      
+      if (rd && rd->response_id == GTK_RESPONSE_CANCEL)
+	{
+	  ret = TRUE;
+	  break;
+	}
+    }
+
+  g_list_free (children);
+
+  return ret;
+}
+
 static void
 gtk_dialog_close (GtkDialog *dialog)
 {
   /* Synthesize delete_event to close dialog. */
   
-  GdkEventAny event;
-  GtkWidget *widget;
+  GtkWidget *widget = GTK_WIDGET (dialog);
+  GdkEvent *event;
 
-  widget = GTK_WIDGET (dialog);
+  if (!dialog_has_cancel (dialog))
+    return;
+
+  event = gdk_event_new (GDK_DELETE);
   
-  event.type = GDK_DELETE;
-  event.window = widget->window;
-  event.send_event = TRUE;
+  event->any.window = g_object_ref (widget->window);
+  event->any.send_event = TRUE;
   
-  g_object_ref (G_OBJECT (event.window));
-  
-  gtk_main_do_event ((GdkEvent*)&event);
-  
-  g_object_unref (G_OBJECT (event.window));
+  gtk_main_do_event (event);
+  gdk_event_free (event);
 }
 
 GtkWidget*
 gtk_dialog_new (void)
 {
-  return GTK_WIDGET (gtk_type_new (GTK_TYPE_DIALOG));
+  return g_object_new (GTK_TYPE_DIALOG, NULL);
 }
 
 static GtkWidget*
@@ -378,7 +426,7 @@ gtk_dialog_new_empty (const gchar     *title,
 {
   GtkDialog *dialog;
 
-  dialog = GTK_DIALOG (g_object_new (GTK_TYPE_DIALOG, NULL));
+  dialog = g_object_new (GTK_TYPE_DIALOG, NULL);
 
   if (title)
     gtk_window_set_title (GTK_WINDOW (dialog), title);
@@ -427,14 +475,14 @@ gtk_dialog_new_empty (const gchar     *title,
  *
  * Here's a simple example:
  * <informalexample><programlisting>
- *  <!>GtkWidget *dialog = gtk_dialog_new_with_buttons ("My dialog",
- *  <!>                                                 main_app_window,
- *  <!>                                                 GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
- *  <!>                                                 GTK_STOCK_OK,
- *  <!>                                                 GTK_RESPONSE_ACCEPT,
- *  <!>                                                 GTK_STOCK_CANCEL,
- *  <!>                                                 GTK_RESPONSE_REJECT,
- *  <!>                                                 NULL);
+ *  GtkWidget *dialog = gtk_dialog_new_with_buttons ("My dialog",
+ *                                                   main_app_window,
+ *                                                   GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+ *                                                   GTK_STOCK_OK,
+ *                                                   GTK_RESPONSE_ACCEPT,
+ *                                                   GTK_STOCK_CANCEL,
+ *                                                   GTK_RESPONSE_REJECT,
+ *                                                   NULL);
  * </programlisting></informalexample>
  * 
  * Return value: a new #GtkDialog
@@ -462,27 +510,21 @@ gtk_dialog_new_with_buttons (const gchar    *title,
   return GTK_WIDGET (dialog);
 }
 
-typedef struct _ResponseData ResponseData;
-
-struct _ResponseData
-{
-  gint response_id;
-};
-
 static ResponseData*
-get_response_data (GtkWidget *widget)
+get_response_data (GtkWidget *widget,
+		   gboolean   create)
 {
-  ResponseData *ad = gtk_object_get_data (GTK_OBJECT (widget),
-                                          "gtk-dialog-response-data");
+  ResponseData *ad = g_object_get_data (G_OBJECT (widget),
+                                        "gtk-dialog-response-data");
 
-  if (ad == NULL)
+  if (ad == NULL && create)
     {
       ad = g_new (ResponseData, 1);
       
-      gtk_object_set_data_full (GTK_OBJECT (widget),
-                                "gtk-dialog-response-data",
-                                ad,
-                                g_free);
+      g_object_set_data_full (G_OBJECT (widget),
+                              "gtk-dialog-response-data",
+                              ad,
+                              g_free);
     }
 
   return ad;
@@ -498,7 +540,7 @@ action_widget_activated (GtkWidget *widget, GtkDialog *dialog)
 
   response_id = GTK_RESPONSE_NONE;
   
-  ad = get_response_data (widget);
+  ad = get_response_data (widget, TRUE);
 
   g_assert (ad != NULL);
   
@@ -506,6 +548,7 @@ action_widget_activated (GtkWidget *widget, GtkDialog *dialog)
 
   gtk_dialog_response (dialog, response_id);
 }
+
 /**
  * gtk_dialog_add_action_widget:
  * @dialog: a #GtkDialog
@@ -520,9 +563,9 @@ action_widget_activated (GtkWidget *widget, GtkDialog *dialog)
  * <literal>action_area</literal> field of the #GtkDialog struct.
  **/
 void
-gtk_dialog_add_action_widget  (GtkDialog *dialog,
-                               GtkWidget *child,
-                               gint       response_id)
+gtk_dialog_add_action_widget (GtkDialog *dialog,
+                              GtkWidget *child,
+                              gint       response_id)
 {
   ResponseData *ad;
   gint signal_id = 0;
@@ -530,26 +573,26 @@ gtk_dialog_add_action_widget  (GtkDialog *dialog,
   g_return_if_fail (GTK_IS_DIALOG (dialog));
   g_return_if_fail (GTK_IS_WIDGET (child));
 
-  ad = get_response_data (child);
+  ad = get_response_data (child, TRUE);
 
   ad->response_id = response_id;
 
   if (GTK_IS_BUTTON (child))
-    {
-      signal_id = g_signal_lookup ("clicked", GTK_TYPE_BUTTON);
-    }
+    signal_id = g_signal_lookup ("clicked", GTK_TYPE_BUTTON);
   else
     signal_id = GTK_WIDGET_GET_CLASS (child)->activate_signal != 0;
 
   if (signal_id)
     {
-      const gchar* name = gtk_signal_name (signal_id);
+      GClosure *closure;
 
-      gtk_signal_connect_while_alive (GTK_OBJECT (child),
-                                      name,
-                                      GTK_SIGNAL_FUNC (action_widget_activated),
-                                      dialog,
-                                      GTK_OBJECT (dialog));
+      closure = g_cclosure_new_object (G_CALLBACK (action_widget_activated),
+				       G_OBJECT (dialog));
+      g_signal_connect_closure_by_id (child,
+				      signal_id,
+				      0,
+				      closure,
+				      FALSE);
     }
   else
     g_warning ("Only 'activatable' widgets can be packed into the action area of a GtkDialog");
@@ -600,9 +643,9 @@ gtk_dialog_add_button (GtkDialog   *dialog,
 }
 
 static void
-gtk_dialog_add_buttons_valist(GtkDialog      *dialog,
-                              const gchar    *first_button_text,
-                              va_list         args)
+gtk_dialog_add_buttons_valist (GtkDialog      *dialog,
+                               const gchar    *first_button_text,
+                               va_list         args)
 {
   const gchar* text;
   gint response_id;
@@ -719,12 +762,7 @@ gtk_dialog_set_default_response (GtkDialog *dialog,
                                             "gtk-dialog-response-data");
 
       if (rd && rd->response_id == response_id)
-	{
-	  gtk_widget_grab_default (widget);
-	  
-	  if (!GTK_WINDOW (dialog)->focus_widget)
-	    gtk_widget_grab_focus (widget);
-	}
+	gtk_widget_grab_default (widget);
 	    
       tmp_list = g_list_next (tmp_list);
     }
@@ -744,10 +782,20 @@ void
 gtk_dialog_set_has_separator (GtkDialog *dialog,
                               gboolean   setting)
 {
+  GtkDialogPrivate *priv;
+
   g_return_if_fail (GTK_IS_DIALOG (dialog));
 
+  priv = GET_PRIVATE (dialog);
+
   /* this might fail if we get called before _init() somehow */
   g_assert (dialog->vbox != NULL);
+
+  if (priv->ignore_separator)
+    {
+      g_warning ("Ignoring the separator setting");
+      return;
+    }
   
   if (setting && dialog->separator == NULL)
     {
@@ -801,9 +849,10 @@ gtk_dialog_response (GtkDialog *dialog,
 {
   g_return_if_fail (GTK_IS_DIALOG (dialog));
 
-  gtk_signal_emit (GTK_OBJECT (dialog),
-                   dialog_signals[RESPONSE],
-                   response_id);
+  g_signal_emit (dialog,
+		 dialog_signals[RESPONSE],
+		 0,
+		 response_id);
 }
 
 typedef struct
@@ -870,10 +919,10 @@ run_destroy_handler (GtkDialog *dialog, gpointer data)
  * @dialog: a #GtkDialog
  * 
  * Blocks in a recursive main loop until the @dialog either emits the
- * response signal, or is destroyed. If the dialog is destroyed,
- * gtk_dialog_run() returns #GTK_RESPONSE_NONE. Otherwise, it returns
- * the response ID from the "response" signal emission. Before
- * entering the recursive main loop, gtk_dialog_run() calls
+ * response signal, or is destroyed. If the dialog is destroyed during the call
+ * to gtk_dialog_run(), gtk_dialog_returns #GTK_RESPONSE_NONE.
+ * Otherwise, it returns the response ID from the "response" signal emission.
+ * Before entering the recursive main loop, gtk_dialog_run() calls
  * gtk_widget_show() on the dialog for you. Note that you still
  * need to show any children of the dialog yourself.
  *
@@ -892,17 +941,17 @@ run_destroy_handler (GtkDialog *dialog, gpointer data)
  *
  * Typical usage of this function might be:
  * <informalexample><programlisting>
- * <!>  gint result = gtk_dialog_run (GTK_DIALOG (dialog));
- * <!>  switch (result)
- * <!>    {
- * <!>      case GTK_RESPONSE_ACCEPT:
- * <!>         do_application_specific_something (<!-- -->);
- * <!>         break;
- * <!>      default:
- * <!>         do_nothing_since_dialog_was_cancelled (<!-- -->);
- * <!>         break;
- * <!>    }
- * <!>  gtk_widget_destroy (dialog);
+ *   gint result = gtk_dialog_run (GTK_DIALOG (dialog));
+ *   switch (result)
+ *     {
+ *       case GTK_RESPONSE_ACCEPT:
+ *          do_application_specific_something (<!-- -->);
+ *          break;
+ *       default:
+ *          do_nothing_since_dialog_was_cancelled (<!-- -->);
+ *          break;
+ *     }
+ *   gtk_widget_destroy (dialog);
  * </programlisting></informalexample>
  * 
  * Return value: response ID
@@ -919,40 +968,40 @@ gtk_dialog_run (GtkDialog *dialog)
   
   g_return_val_if_fail (GTK_IS_DIALOG (dialog), -1);
 
-  gtk_object_ref (GTK_OBJECT (dialog));
+  g_object_ref (dialog);
 
-  if (!GTK_WIDGET_VISIBLE (dialog))
-    gtk_widget_show (GTK_WIDGET (dialog));
-  
   was_modal = GTK_WINDOW (dialog)->modal;
   if (!was_modal)
     gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
 
+  if (!GTK_WIDGET_VISIBLE (dialog))
+    gtk_widget_show (GTK_WIDGET (dialog));
+  
   response_handler =
-    gtk_signal_connect (GTK_OBJECT (dialog),
-                        "response",
-                        GTK_SIGNAL_FUNC (run_response_handler),
-                        &ri);
+    g_signal_connect (dialog,
+                      "response",
+                      G_CALLBACK (run_response_handler),
+                      &ri);
   
   unmap_handler =
-    gtk_signal_connect (GTK_OBJECT (dialog),
-                        "unmap",
-                        GTK_SIGNAL_FUNC (run_unmap_handler),
-                        &ri);
+    g_signal_connect (dialog,
+                      "unmap",
+                      G_CALLBACK (run_unmap_handler),
+                      &ri);
   
   delete_handler =
-    gtk_signal_connect (GTK_OBJECT (dialog),
-                        "delete_event",
-                        GTK_SIGNAL_FUNC (run_delete_handler),
-                        &ri);
+    g_signal_connect (dialog,
+                      "delete_event",
+                      G_CALLBACK (run_delete_handler),
+                      &ri);
   
   destroy_handler =
-    gtk_signal_connect (GTK_OBJECT (dialog),
-                        "destroy",
-                        GTK_SIGNAL_FUNC (run_destroy_handler),
-                        &ri);
+    g_signal_connect (dialog,
+                      "destroy",
+                      G_CALLBACK (run_destroy_handler),
+                      &ri);
   
-  ri.loop = g_main_new (FALSE);
+  ri.loop = g_main_loop_new (NULL, FALSE);
 
   GDK_THREADS_LEAVE ();  
   g_main_loop_run (ri.loop);
@@ -968,13 +1017,36 @@ gtk_dialog_run (GtkDialog *dialog)
       if (!was_modal)
         gtk_window_set_modal (GTK_WINDOW(dialog), FALSE);
       
-      gtk_signal_disconnect (GTK_OBJECT (dialog), response_handler);
-      gtk_signal_disconnect (GTK_OBJECT (dialog), unmap_handler);
-      gtk_signal_disconnect (GTK_OBJECT (dialog), delete_handler);
-      gtk_signal_disconnect (GTK_OBJECT (dialog), destroy_handler);
+      g_signal_handler_disconnect (dialog, response_handler);
+      g_signal_handler_disconnect (dialog, unmap_handler);
+      g_signal_handler_disconnect (dialog, delete_handler);
+      g_signal_handler_disconnect (dialog, destroy_handler);
     }
 
-  gtk_object_unref (GTK_OBJECT (dialog));
+  g_object_unref (dialog);
 
   return ri.response_id;
 }
+
+void
+_gtk_dialog_set_ignore_separator (GtkDialog *dialog,
+				  gboolean   ignore_separator)
+{
+  GtkDialogPrivate *priv;
+
+  priv = GET_PRIVATE (dialog);
+  priv->ignore_separator = ignore_separator;
+}
+
+gint
+_gtk_dialog_get_response_for_widget (GtkDialog *dialog,
+				     GtkWidget *widget)
+{
+  ResponseData *rd;
+
+  rd = get_response_data (widget, FALSE);
+  if (!rd)
+    return GTK_RESPONSE_NONE;
+  else
+    return rd->response_id;
+}
