@@ -42,6 +42,8 @@ struct _GtkFileChooserEntry
 {
   GtkEntry parent_instance;
 
+  GtkFileChooserAction action;
+
   GtkFileSystem *file_system;
   GtkFilePath *base_folder;
   GtkFilePath *current_folder_path;
@@ -99,7 +101,6 @@ static char    *maybe_append_separator_to_path (GtkFileChooserEntry *chooser_ent
 						GtkFilePath         *path,
 						gchar               *display_name);
 
-
 static GObjectClass *parent_class;
 static GtkEditableClass *parent_editable_iface;
 
@@ -174,6 +175,7 @@ gtk_file_chooser_entry_init (GtkFileChooserEntry *chooser_entry)
   GtkCellRenderer *cell;
 
   comp = gtk_entry_completion_new ();
+
   gtk_entry_completion_set_match_func (comp,
 				       completion_match_func,
 				       chooser_entry,
@@ -317,6 +319,20 @@ completion_match_func (GtkEntryCompletion *comp,
   norm_file_part = g_utf8_normalize (chooser_entry->file_part, -1, G_NORMALIZE_ALL);
   norm_name = g_utf8_normalize (name, -1, G_NORMALIZE_ALL);
 
+#ifdef G_PLATFORM_WIN32
+  {
+    gchar *temp;
+
+    temp = norm_file_part;
+    norm_file_part = g_utf8_casefold (norm_file_part, -1);
+    g_free (temp);
+
+    temp = norm_name;
+    norm_name = g_utf8_casefold (norm_name, -1);
+    g_free (temp);
+  }
+#endif
+
   result = (strncmp (norm_file_part, norm_name, strlen (norm_file_part)) == 0);
 
   g_free (norm_file_part);
@@ -326,10 +342,11 @@ completion_match_func (GtkEntryCompletion *comp,
   return result;
 }
 
-/* This function will append a '/' character to paths to display_name iff the
- * path associated with it is a directory.  maybe_append_separator_to_path will
- * g_free the display_name and return a new one if needed.  Otherwise, it will
- * return the old one.  You should be safe calling
+/* This function will append a directory separator to paths to
+ * display_name iff the path associated with it is a directory.
+ * maybe_append_separator_to_path will g_free the display_name and
+ * return a new one if needed.  Otherwise, it will return the old one.
+ * You should be safe calling
  *
  * display_name = maybe_append_separator_to_path (entry, path, display_name);
  * ...
@@ -352,7 +369,7 @@ maybe_append_separator_to_path (GtkFileChooserEntry *chooser_entry,
 	  if (gtk_file_info_get_is_folder (info))
 	    {
 	      gchar *tmp = display_name;
-	      display_name = g_strconcat (tmp, "/", NULL);
+	      display_name = g_strconcat (tmp, G_DIR_SEPARATOR_S, NULL);
 	      g_free (tmp);
 	    }
 	  
@@ -435,6 +452,19 @@ check_completion_callback (GtkFileChooserEntry *chooser_entry)
       gtk_file_path_free (unique_path);
     }
 
+  switch (chooser_entry->action)
+    {
+    case GTK_FILE_CHOOSER_ACTION_SAVE:
+    case GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER:
+      if (common_prefix && !g_str_has_suffix (common_prefix, "/"))
+	{
+	  g_free (common_prefix);
+	  common_prefix = NULL;
+	}
+      break;
+    default: ;
+    }
+
   if (common_prefix)
     {
       gint file_part_len;
@@ -574,19 +604,22 @@ load_directory_callback (GtkFileChooserEntry *chooser_entry)
   g_signal_connect (chooser_entry->current_folder, "files-removed",
 		    G_CALLBACK (files_deleted_cb), chooser_entry);
   
-  gtk_file_folder_list_children (chooser_entry->current_folder,
-				 &child_paths,
-				 NULL); /* NULL-GError */
   chooser_entry->completion_store = gtk_list_store_new (N_COLUMNS,
 							G_TYPE_STRING,
 							GTK_TYPE_FILE_PATH);
 
-  if (child_paths)
+  if (chooser_entry->file_part_pos != -1)
     {
-      update_current_folder_files (chooser_entry, child_paths);
-      add_completion_idle (chooser_entry);
-      gtk_file_paths_free (child_paths);
-  }
+      gtk_file_folder_list_children (chooser_entry->current_folder,
+				     &child_paths,
+				     NULL); /* NULL-GError */
+      if (child_paths)
+	{
+	  update_current_folder_files (chooser_entry, child_paths);
+	  add_completion_idle (chooser_entry);
+	  gtk_file_paths_free (child_paths);
+	}
+    }
 
   gtk_entry_completion_set_model (gtk_entry_get_completion (GTK_ENTRY (chooser_entry)),
 				  GTK_TREE_MODEL (chooser_entry->completion_store));
@@ -657,13 +690,14 @@ gtk_file_chooser_entry_activate (GtkEntry *entry)
  */
 static void
 gtk_file_chooser_entry_maybe_update_directory (GtkFileChooserEntry *chooser_entry,
-					       GtkFilePath         *folder_path)
+					       GtkFilePath         *folder_path,
+					       gboolean             force_reload)
 {
   gboolean queue_idle = FALSE;
 
   if (chooser_entry->current_folder_path)
     {
-      if (gtk_file_path_compare (folder_path, chooser_entry->current_folder_path) != 0)
+      if (gtk_file_path_compare (folder_path, chooser_entry->current_folder_path) != 0 || force_reload)
 	{
 	  /* We changed our current directory.  We need to clear out the old
 	   * directory information.
@@ -736,22 +770,25 @@ gtk_file_chooser_entry_changed (GtkEditable *editable)
     {
       folder_path = gtk_file_path_copy (chooser_entry->base_folder);
       file_part = g_strdup ("");
+      file_part_pos = -1;
     }
-
-  file_part_len = strlen (file_part);
-  total_len = strlen (text);
-  if (total_len > file_part_len)
-    file_part_pos = g_utf8_strlen (text, total_len - file_part_len);
   else
-    file_part_pos = 0;
-
-  gtk_file_chooser_entry_maybe_update_directory (chooser_entry, folder_path);
+    {
+      file_part_len = strlen (file_part);
+      total_len = strlen (text);
+      if (total_len > file_part_len)
+	file_part_pos = g_utf8_strlen (text, total_len - file_part_len);
+      else
+	file_part_pos = 0;
+    }
 
   if (chooser_entry->file_part)
     g_free (chooser_entry->file_part);
 
   chooser_entry->file_part = file_part;
   chooser_entry->file_part_pos = file_part_pos;
+
+  gtk_file_chooser_entry_maybe_update_directory (chooser_entry, folder_path, file_part_pos == -1);
 }
 
 static void
@@ -820,6 +857,7 @@ _gtk_file_chooser_entry_set_base_folder (GtkFileChooserEntry *chooser_entry,
 
   chooser_entry->base_folder = gtk_file_path_copy (path);
 
+  gtk_file_chooser_entry_changed (GTK_EDITABLE (chooser_entry));
   gtk_editable_select_region (GTK_EDITABLE (chooser_entry), 0, -1);
 }
 
@@ -841,6 +879,11 @@ _gtk_file_chooser_entry_set_base_folder (GtkFileChooserEntry *chooser_entry,
 const GtkFilePath *
 _gtk_file_chooser_entry_get_current_folder (GtkFileChooserEntry *chooser_entry)
 {
+  if (chooser_entry->has_completion)
+    {
+      gtk_editable_set_position (GTK_EDITABLE (chooser_entry),
+				 GTK_ENTRY (chooser_entry)->text_length);
+    }
   return chooser_entry->current_folder_path;
 }
 
@@ -859,6 +902,11 @@ _gtk_file_chooser_entry_get_current_folder (GtkFileChooserEntry *chooser_entry)
 const gchar *
 _gtk_file_chooser_entry_get_file_part (GtkFileChooserEntry *chooser_entry)
 {
+  if (chooser_entry->has_completion)
+    {
+      gtk_editable_set_position (GTK_EDITABLE (chooser_entry),
+				 GTK_ENTRY (chooser_entry)->text_length);
+    }
   return chooser_entry->file_part;
 }
 
@@ -877,3 +925,43 @@ _gtk_file_chooser_entry_set_file_part (GtkFileChooserEntry *chooser_entry,
 
   gtk_entry_set_text (GTK_ENTRY (chooser_entry), file_part);
 }
+
+
+/**
+ * _gtk_file_chooser_entry_set_action:
+ * @chooser_entry: a #GtkFileChooserEntry
+ * @action: the action which is performed by the file selector using this entry
+ *
+ * Sets action which is performed by the file selector using this entry. 
+ * The #GtkFileChooserEntry will use different completion strategies for 
+ * different actions.
+ **/
+void
+_gtk_file_chooser_entry_set_action (GtkFileChooserEntry *chooser_entry,
+				    GtkFileChooserAction action)
+{
+  g_return_if_fail (GTK_IS_FILE_CHOOSER_ENTRY (chooser_entry));
+  
+  if (  chooser_entry->action != action)
+    {
+      chooser_entry->action = action;
+    }
+}
+
+
+/**
+ * _gtk_file_chooser_entry_get_action:
+ * @chooser_entry: a #GtkFileChooserEntry
+ *
+ * Gets the action for this entry. 
+ *
+ * Returns: the action
+ **/
+GtkFileChooserAction
+_gtk_file_chooser_entry_get_action (GtkFileChooserEntry *chooser_entry)
+{
+  g_return_val_if_fail (GTK_IS_FILE_CHOOSER_ENTRY (chooser_entry),
+			GTK_FILE_CHOOSER_ACTION_OPEN);
+  
+  return chooser_entry->action;
+}
