@@ -153,6 +153,7 @@ enum
   PROP_UI
 };
 
+static GObjectClass *parent_class = NULL;
 static guint ui_manager_signals[LAST_SIGNAL] = { 0 };
 
 static GMemChunk *merge_node_chunk = NULL;
@@ -190,6 +191,8 @@ gtk_ui_manager_class_init (GtkUIManagerClass *klass)
 {
   GObjectClass *gobject_class;
 
+  parent_class = g_type_class_peek_parent (klass);
+
   gobject_class = G_OBJECT_CLASS (klass);
 
   if (!merge_node_chunk)
@@ -243,7 +246,8 @@ gtk_ui_manager_class_init (GtkUIManagerClass *klass)
     g_signal_new ("add_widget",
 		  G_OBJECT_CLASS_TYPE (klass),
 		  G_SIGNAL_RUN_FIRST | G_SIGNAL_NO_RECURSE,
-		  G_STRUCT_OFFSET (GtkUIManagerClass, add_widget), NULL, NULL,
+		  G_STRUCT_OFFSET (GtkUIManagerClass, add_widget),
+		  NULL, NULL,
 		  g_cclosure_marshal_VOID__OBJECT,
 		  G_TYPE_NONE, 1, 
 		  GTK_TYPE_WIDGET);
@@ -267,7 +271,7 @@ gtk_ui_manager_class_init (GtkUIManagerClass *klass)
 		  G_TYPE_NONE, 0);
   
   /**
-   * GtkUIManager::connect_proxy:
+   * GtkUIManager::connect-proxy:
    * @uimanager: the ui manager
    * @action: the action
    * @proxy: the proxy
@@ -284,13 +288,16 @@ gtk_ui_manager_class_init (GtkUIManagerClass *klass)
   ui_manager_signals[CONNECT_PROXY] =
     g_signal_new ("connect_proxy",
 		  G_OBJECT_CLASS_TYPE (klass),
-		  0, 0, NULL, NULL,
+		  G_SIGNAL_RUN_FIRST | G_SIGNAL_NO_RECURSE,
+		  G_STRUCT_OFFSET (GtkUIManagerClass, connect_proxy),
+		  NULL, NULL,
 		  _gtk_marshal_VOID__OBJECT_OBJECT,
 		  G_TYPE_NONE, 2, 
-		  GTK_TYPE_ACTION, GTK_TYPE_WIDGET);
+		  GTK_TYPE_ACTION,
+		  GTK_TYPE_WIDGET);
 
   /**
-   * GtkUIManager::disconnect_proxy:
+   * GtkUIManager::disconnect-proxy:
    * @uimanager: the ui manager
    * @action: the action
    * @proxy: the proxy
@@ -303,13 +310,16 @@ gtk_ui_manager_class_init (GtkUIManagerClass *klass)
   ui_manager_signals[DISCONNECT_PROXY] =
     g_signal_new ("disconnect_proxy",
 		  G_OBJECT_CLASS_TYPE (klass),
-		  0, 0, NULL, NULL,
+		  G_SIGNAL_RUN_FIRST | G_SIGNAL_NO_RECURSE,
+		  G_STRUCT_OFFSET (GtkUIManagerClass, disconnect_proxy),
+		  NULL, NULL,
 		  _gtk_marshal_VOID__OBJECT_OBJECT,
-		  G_TYPE_NONE, 2, 
-		  GTK_TYPE_ACTION, GTK_TYPE_WIDGET);
+		  G_TYPE_NONE, 2,
+		  GTK_TYPE_ACTION,
+		  GTK_TYPE_WIDGET);
 
   /**
-   * GtkUIManager::pre_activate:
+   * GtkUIManager::pre-activate:
    * @uimanager: the ui manager
    * @action: the action
    *
@@ -324,13 +334,15 @@ gtk_ui_manager_class_init (GtkUIManagerClass *klass)
   ui_manager_signals[PRE_ACTIVATE] =
     g_signal_new ("pre_activate",
 		  G_OBJECT_CLASS_TYPE (klass),
-		  0, 0, NULL, NULL,
+		  G_SIGNAL_RUN_FIRST | G_SIGNAL_NO_RECURSE,
+		  G_STRUCT_OFFSET (GtkUIManagerClass, pre_activate),
+		  NULL, NULL,
 		  _gtk_marshal_VOID__OBJECT,
-		  G_TYPE_NONE, 1, 
+		  G_TYPE_NONE, 1,
 		  GTK_TYPE_ACTION);
 
   /**
-   * GtkUIManager::post_activate:
+   * GtkUIManager::post-activate:
    * @uimanager: the ui manager
    * @action: the action
    *
@@ -345,11 +357,20 @@ gtk_ui_manager_class_init (GtkUIManagerClass *klass)
   ui_manager_signals[POST_ACTIVATE] =
     g_signal_new ("post_activate",
 		  G_OBJECT_CLASS_TYPE (klass),
-		  0, 0, NULL, NULL,
+		  G_SIGNAL_RUN_FIRST | G_SIGNAL_NO_RECURSE,
+		  G_STRUCT_OFFSET (GtkUIManagerClass, post_activate),
+		  NULL, NULL,
 		  _gtk_marshal_VOID__OBJECT,
-		  G_TYPE_NONE, 1, 
+		  G_TYPE_NONE, 1,
 		  GTK_TYPE_ACTION);
 
+  klass->add_widget = NULL;
+  klass->actions_changed = NULL;
+  klass->connect_proxy = NULL;
+  klass->disconnect_proxy = NULL;
+  klass->pre_activate = NULL;
+  klass->post_activate = NULL;
+
   g_type_class_add_private (gobject_class, sizeof (GtkUIManagerPrivate));
 }
 
@@ -400,6 +421,8 @@ gtk_ui_manager_finalize (GObject *object)
 
   g_object_unref (self->private_data->accel_group);
   self->private_data->accel_group = NULL;
+
+  G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
 static void
@@ -574,7 +597,8 @@ gtk_ui_manager_insert_action_group (GtkUIManager   *self,
 		    "object_signal::connect_proxy", G_CALLBACK (cb_proxy_connect_proxy), self,
 		    "object_signal::disconnect_proxy", G_CALLBACK (cb_proxy_disconnect_proxy), self,
 		    "object_signal::pre_activate", G_CALLBACK (cb_proxy_pre_activate), self,
-		    "object_signal::post_activate", G_CALLBACK (cb_proxy_post_activate), self, 0);
+		    "object_signal::post_activate", G_CALLBACK (cb_proxy_post_activate), self,
+		    NULL);
 
   /* dirty all nodes, as action bindings may change */
   dirty_all_nodes (self);
@@ -609,7 +633,7 @@ gtk_ui_manager_remove_action_group (GtkUIManager   *self,
                        "any_signal::disconnect_proxy", G_CALLBACK (cb_proxy_disconnect_proxy), self,
                        "any_signal::pre_activate", G_CALLBACK (cb_proxy_pre_activate), self,
                        "any_signal::post_activate", G_CALLBACK (cb_proxy_post_activate), self, 
-                       0);
+                       NULL);
   g_object_unref (action_group);
 
   /* dirty all nodes, as action bindings may change */
@@ -666,6 +690,9 @@ gtk_ui_manager_get_accel_group (GtkUIManager *self)
  * the XML (e.g. &lt;popup&gt;) can be addressed by their XML element name 
  * (e.g. "popup"). The root element ("/ui") can be omitted in the path.
  * 
+ * Note that the widget found by following a path that ends in a &lt;menu&gt;
+ * element is the menuitem to which the menu is attached, not the menu itself.
+ * 
  * Return value: the widget found by following the path, or %NULL if no widget
  *   was found.
  *
@@ -813,7 +840,8 @@ get_child_node (GtkUIManager *self,
 	{
 	  for (child = parent->children; child != NULL; child = child->next)
 	    {
-	      if (strlen (NODE_INFO (child)->name) == childname_length &&
+	      if (NODE_INFO (child)->name &&
+		  strlen (NODE_INFO (child)->name) == childname_length &&
 		  !strncmp (NODE_INFO (child)->name, childname, childname_length))
 		{
 		  /* if undecided about node type, set it */
@@ -885,6 +913,9 @@ get_node (GtkUIManager *self,
   const gchar *pos, *end;
   GNode *parent, *node;
   
+  if (strncmp ("/ui", path, 3) == 0)
+    path += 3;
+
   end = path + strlen (path);
   pos = path;
   parent = node = NULL;
@@ -924,6 +955,10 @@ free_node (GNode *node)
 
   if (info->action)
     g_object_unref (info->action);
+  if (info->proxy)
+    g_object_unref (info->proxy);
+  if (info->extra)
+    g_object_unref (info->extra);
   g_free (info->name);
   g_chunk_free (info, merge_node_chunk);
 
@@ -978,7 +1013,7 @@ node_remove_ui_reference (Node  *node,
       
       if (reference->merge_id == merge_id)
 	{
-	  node->uifiles = g_list_remove_link (node->uifiles, p);
+	  node->uifiles = g_list_delete_link (node->uifiles, p);
 	  node->dirty = TRUE;
 	  g_free (reference);
 
@@ -1204,15 +1239,24 @@ start_element_handler (GMarkupParseContext *context,
 	  !strcmp (element_name, "separator"))
 	{
 	  GNode *node;
+	  gint length;
 
 	  if (ctx->state == STATE_TOOLBAR)
 	    ctx->state = STATE_TOOLITEM;
 	  else
 	    ctx->state = STATE_MENUITEM;
+	  if (!strcmp (node_name, "separator"))
+	    {
+	      node_name = NULL;
+	      length = -1;
+	    }
+	  else
+	    length = strlen (node_name);
 	  node = get_child_node (self, ctx->current,
-				 node_name, strlen (node_name),
+				 node_name, length,
 				 NODE_TYPE_SEPARATOR,
 				 TRUE, top);
+
 	  if (NODE_INFO (node)->action_name == 0)
 	    NODE_INFO (node)->action_name = action_quark;
 
@@ -1389,17 +1433,17 @@ add_ui_from_string (GtkUIManager *self,
 
   if (needs_root)
     if (!g_markup_parse_context_parse (context, "<ui>", -1, error))
-      goto error;
+      goto out;
 
   if (!g_markup_parse_context_parse (context, buffer, length, error))
-    goto error;
+    goto out;
 
   if (needs_root)
     if (!g_markup_parse_context_parse (context, "</ui>", -1, error))
-      goto error;
+      goto out;
 
   if (!g_markup_parse_context_end_parse (context, error))
-    goto error;
+    goto out;
 
   g_markup_parse_context_free (context);
 
@@ -1409,7 +1453,7 @@ add_ui_from_string (GtkUIManager *self,
 
   return ctx.merge_id;
 
- error:
+ out:
 
   g_markup_parse_context_free (context);
 
@@ -1587,7 +1631,7 @@ gtk_ui_manager_add_ui (GtkUIManager        *self,
 	  node_type = NODE_TYPE_SEPARATOR;
 	  break;
 	case GTK_UI_MANAGER_PLACEHOLDER:
-	  node_type = NODE_TYPE_MENU_PLACEHOLDER;
+	  node_type = NODE_TYPE_TOOLBAR_PLACEHOLDER;
 	  break;
 	default: ;
 	  /* do nothing */
@@ -2085,6 +2129,8 @@ update_node (GtkUIManager *self,
 	  if (info->proxy == NULL)
 	    {
 	      info->proxy = gtk_menu_bar_new ();
+	      g_object_ref (info->proxy);
+	      gtk_object_sink (GTK_OBJECT (info->proxy));
 	      gtk_widget_set_name (info->proxy, info->name);
 	      gtk_widget_show (info->proxy);
 	      g_signal_emit (self, ui_manager_signals[ADD_WIDGET], 0, info->proxy);
@@ -2092,7 +2138,11 @@ update_node (GtkUIManager *self,
 	  break;
 	case NODE_TYPE_POPUP:
 	  if (info->proxy == NULL) 
-	    info->proxy = gtk_menu_new ();
+	    {
+	      info->proxy = gtk_menu_new ();
+	      g_object_ref (info->proxy);
+	      gtk_object_sink (GTK_OBJECT (info->proxy));
+	    }
 	  gtk_widget_set_name (info->proxy, info->name);
 	  break;
 	case NODE_TYPE_MENU:
@@ -2113,6 +2163,7 @@ update_node (GtkUIManager *self,
 		gtk_action_disconnect_proxy (info->action, info->proxy);
 		gtk_container_remove (GTK_CONTAINER (info->proxy->parent),
 				      info->proxy);
+		g_object_unref (info->proxy);
 		info->proxy = NULL;
 	      }
 	    /* create proxy if needed ... */
@@ -2127,6 +2178,8 @@ update_node (GtkUIManager *self,
 		    GtkWidget *filler;
 
 		    info->proxy = gtk_action_create_menu_item (action);
+		    g_object_ref (info->proxy);
+		    gtk_object_sink (GTK_OBJECT (info->proxy));
 		    menu = gtk_menu_new ();
 		    gtk_widget_set_name (info->proxy, info->name);
 		    gtk_widget_set_name (menu, info->name);
@@ -2171,6 +2224,8 @@ update_node (GtkUIManager *self,
 	  if (info->proxy == NULL)
 	    {
 	      info->proxy = gtk_toolbar_new ();
+	      g_object_ref (info->proxy);
+	      gtk_object_sink (GTK_OBJECT (info->proxy));
 	      gtk_widget_set_name (info->proxy, info->name);
 	      gtk_widget_show (info->proxy);
 	      g_signal_emit (self, ui_manager_signals[ADD_WIDGET], 0, info->proxy);
@@ -2185,12 +2240,14 @@ update_node (GtkUIManager *self,
 		{
 		  gtk_container_remove (GTK_CONTAINER (info->proxy->parent),
 					info->proxy);
+		  g_object_unref (info->proxy);
 		  info->proxy = NULL;
 		}
 	      if (info->extra)
 		{
 		  gtk_container_remove (GTK_CONTAINER (info->extra->parent),
 					info->extra);
+		  g_object_unref (info->extra);
 		  info->extra = NULL;
 		}
 	    }
@@ -2202,16 +2259,22 @@ update_node (GtkUIManager *self,
 	      if (find_menu_position (node, &menushell, &pos))
 		{
 		  info->proxy = gtk_separator_menu_item_new ();
+		  g_object_ref (info->proxy);
+		  gtk_object_sink (GTK_OBJECT (info->proxy));
 		  g_object_set_data (G_OBJECT (info->proxy),
 				     "gtk-separator-mode",
 				     GINT_TO_POINTER (SEPARATOR_MODE_HIDDEN));
+		  gtk_widget_set_no_show_all (info->proxy, TRUE);
 		  gtk_menu_shell_insert (GTK_MENU_SHELL (menushell),
 					NODE_INFO (node)->proxy, pos);
 
 		  info->extra = gtk_separator_menu_item_new ();
+		  g_object_ref (info->extra);
+		  gtk_object_sink (GTK_OBJECT (info->extra));
 		  g_object_set_data (G_OBJECT (info->extra),
 				     "gtk-separator-mode",
 				     GINT_TO_POINTER (SEPARATOR_MODE_HIDDEN));
+		  gtk_widget_set_no_show_all (info->extra, TRUE);
 		  gtk_menu_shell_insert (GTK_MENU_SHELL (menushell),
 					 NODE_INFO (node)->extra, pos+1);
 		}
@@ -2226,12 +2289,14 @@ update_node (GtkUIManager *self,
 		{
 		  gtk_container_remove (GTK_CONTAINER (info->proxy->parent),
 					info->proxy);
+		  g_object_unref (info->proxy);
 		  info->proxy = NULL;
 		}
 	      if (info->extra)
 		{
 		  gtk_container_remove (GTK_CONTAINER (info->extra->parent),
 					info->extra);
+		  g_object_unref (info->extra);
 		  info->extra = NULL;
 		}
 	    }
@@ -2247,16 +2312,22 @@ update_node (GtkUIManager *self,
 		  item = gtk_separator_tool_item_new ();
 		  gtk_toolbar_insert (GTK_TOOLBAR (toolbar), item, pos);
 		  info->proxy = GTK_WIDGET (item);
+		  g_object_ref (info->proxy);
+		  gtk_object_sink (GTK_OBJECT (info->proxy));
 		  g_object_set_data (G_OBJECT (info->proxy),
 				     "gtk-separator-mode",
 				     GINT_TO_POINTER (SEPARATOR_MODE_HIDDEN));
+		  gtk_widget_set_no_show_all (info->proxy, TRUE);
 
 		  item = gtk_separator_tool_item_new ();
 		  gtk_toolbar_insert (GTK_TOOLBAR (toolbar), item, pos+1);
 		  info->extra = GTK_WIDGET (item);
+		  g_object_ref (info->extra);
+		  gtk_object_sink (GTK_OBJECT (info->extra));
 		  g_object_set_data (G_OBJECT (info->extra),
 				     "gtk-separator-mode",
 				     GINT_TO_POINTER (SEPARATOR_MODE_HIDDEN));
+		  gtk_widget_set_no_show_all (info->extra, TRUE);
 		}
 	    }
 	  break;
@@ -2271,6 +2342,7 @@ update_node (GtkUIManager *self,
 	      gtk_action_disconnect_proxy (info->action, info->proxy);
 	      gtk_container_remove (GTK_CONTAINER (info->proxy->parent),
 				    info->proxy);
+	      g_object_unref (info->proxy);
 	      info->proxy = NULL;
 	    }
 	  /* create proxy if needed ... */
@@ -2282,6 +2354,8 @@ update_node (GtkUIManager *self,
 	      if (find_menu_position (node, &menushell, &pos))
 		{
 		  info->proxy = gtk_action_create_menu_item (action);
+		  g_object_ref (info->proxy);
+		  gtk_object_sink (GTK_OBJECT (info->proxy));
 		  gtk_widget_set_name (info->proxy, info->name);
 		  
 		  gtk_menu_shell_insert (GTK_MENU_SHELL (menushell),
@@ -2310,8 +2384,8 @@ update_node (GtkUIManager *self,
 	  break;
 	case NODE_TYPE_TOOLITEM:
 	  /* remove the proxy if it is of the wrong type ... */
-	  if (info->proxy &&  G_OBJECT_TYPE (info->proxy) !=
-	      GTK_ACTION_GET_CLASS (action)->toolbar_item_type)
+	  if (info->proxy &&  
+	      G_OBJECT_TYPE (info->proxy) != GTK_ACTION_GET_CLASS (action)->toolbar_item_type)
 	    {
 	      g_signal_handlers_disconnect_by_func (info->proxy,
 						    G_CALLBACK (update_smart_separators),
@@ -2319,6 +2393,7 @@ update_node (GtkUIManager *self,
 	      gtk_action_disconnect_proxy (info->action, info->proxy);
 	      gtk_container_remove (GTK_CONTAINER (info->proxy->parent),
 				    info->proxy);
+	      g_object_unref (info->proxy);
 	      info->proxy = NULL;
 	    }
 	  /* create proxy if needed ... */
@@ -2330,6 +2405,8 @@ update_node (GtkUIManager *self,
 	      if (find_toolbar_position (node, &toolbar, &pos))
 		{
 		  info->proxy = gtk_action_create_tool_item (action);
+		  g_object_ref (info->proxy);
+		  gtk_object_sink (GTK_OBJECT (info->proxy));
 		  gtk_widget_set_name (info->proxy, info->name);
 
 		  gtk_toolbar_insert (GTK_TOOLBAR (toolbar),
@@ -2365,6 +2442,7 @@ update_node (GtkUIManager *self,
 		{
 		  gtk_container_remove (GTK_CONTAINER (info->proxy->parent),
 					info->proxy);
+ 		  g_object_unref (info->proxy);
 		  info->proxy = NULL;
 		}
 
@@ -2373,6 +2451,8 @@ update_node (GtkUIManager *self,
 		  GtkToolItem *item = gtk_separator_tool_item_new ();
 		  gtk_toolbar_insert (GTK_TOOLBAR (toolbar), item, pos);
 		  info->proxy = GTK_WIDGET (item);
+		  g_object_ref (info->proxy);
+		  gtk_object_sink (GTK_OBJECT (info->proxy));
 		  gtk_widget_set_no_show_all (info->proxy, TRUE);
 		  g_object_set_data (G_OBJECT (info->proxy),
 				     "gtk-separator-mode",
@@ -2389,12 +2469,15 @@ update_node (GtkUIManager *self,
 		{
 		  gtk_container_remove (GTK_CONTAINER (info->proxy->parent),
 					info->proxy);
+		  g_object_unref (info->proxy);
 		  info->proxy = NULL;
 		}
 
 	      if (find_menu_position (node, &menushell, &pos))
 		{
 		  info->proxy = gtk_separator_menu_item_new ();
+		  g_object_ref (info->proxy);
+		  gtk_object_sink (GTK_OBJECT (info->proxy));
 		  gtk_widget_set_no_show_all (info->proxy, TRUE);
 		  g_object_set_data (G_OBJECT (info->proxy),
 				     "gtk-separator-mode",
@@ -2454,8 +2537,6 @@ update_node (GtkUIManager *self,
 static gboolean
 do_updates (GtkUIManager *self)
 {
-  GDK_THREADS_ENTER ();
-
   /* this function needs to check through the tree for dirty nodes.
    * For such nodes, it needs to do the following:
    *
@@ -2472,6 +2553,14 @@ do_updates (GtkUIManager *self)
 
   self->private_data->update_tag = 0;
 
+  return FALSE;
+}
+
+static gboolean
+do_updates_idle (GtkUIManager *self)
+{
+  GDK_THREADS_ENTER ();
+  do_updates (self);
   GDK_THREADS_LEAVE ();
 
   return FALSE;
@@ -2483,7 +2572,8 @@ queue_update (GtkUIManager *self)
   if (self->private_data->update_tag != 0)
     return;
 
-  self->private_data->update_tag = g_idle_add ((GSourceFunc)do_updates, self);
+  self->private_data->update_tag = g_idle_add ((GSourceFunc)do_updates_idle, 
+					       self);
 }
 
 
@@ -2522,7 +2612,7 @@ gtk_ui_manager_ensure_update (GtkUIManager *self)
 }
 
 static gboolean
-dirty_traverse_func (GNode   *node, 
+dirty_traverse_func (GNode   *node,
 		     gpointer data)
 {
   NODE_INFO (node)->dirty = TRUE;
@@ -2532,25 +2622,25 @@ dirty_traverse_func (GNode   *node,
 static void
 dirty_all_nodes (GtkUIManager *self)
 {
-  g_node_traverse (self->private_data->root_node, 
+  g_node_traverse (self->private_data->root_node,
 		   G_PRE_ORDER, G_TRAVERSE_ALL, -1,
 		   dirty_traverse_func, NULL);
   queue_update (self);
 }
 
 static const gchar *open_tag_format[] = {
-  "%*s<UNDECIDED>\n",
-  "%*s<ui>\n",
-  "%*s<menubar name=\"%s\">\n",  
-  "%*s<menu name=\"%s\" action=\"%s\">\n",
-  "%*s<toolbar name=\"%s\">\n",
-  "%*s<placeholder name=\"%s\">\n",
-  "%*s<placeholder name=\"%s\">\n",
-  "%*s<popup name=\"%s\">\n",
-  "%*s<menuitem name=\"%s\" action=\"%s\"/>\n", 
-  "%*s<toolitem name=\"%s\" action=\"%s\"/>\n", 
-  "%*s<separator name=\"%s\"/>\n",
-  "%*s<accelerator name=\"%s\" action=\"%s\"/>\n",
+  "%*s<UNDECIDED",
+  "%*s<ui",
+  "%*s<menubar",
+  "%*s<menu",
+  "%*s<toolbar",
+  "%*s<placeholder",
+  "%*s<placeholder",
+  "%*s<popup",
+  "%*s<menuitem",
+  "%*s<toolitem",
+  "%*s<separator",
+  "%*s<accelerator"
 };
 
 static const gchar *close_tag_format[] = {
@@ -2562,34 +2652,45 @@ static const gchar *close_tag_format[] = {
   "%*s</placeholder>\n",
   "%*s</placeholder>\n",
   "%*s</popup>\n",
-  "",
-  "",
-  "",
-  "",
+  NULL,
+  NULL,
+  NULL,
+  NULL
 };
 
 static void
-print_node (GtkUIManager *self, 
-	    GNode        *node, 
+print_node (GtkUIManager *self,
+	    GNode        *node,
 	    gint          indent_level,
 	    GString      *buffer)
 {
-  Node *mnode;
+  Node  *mnode;
   GNode *child;
 
   mnode = node->data;
 
   g_string_append_printf (buffer, open_tag_format[mnode->type],
-			  indent_level, "",
-			  mnode->name, 
-			  g_quark_to_string (mnode->action_name));
-
-  for (child = node->children; child != NULL; child = child->next) 
-    print_node (self, child, indent_level + 2, buffer);
-
-  g_string_append_printf (buffer, close_tag_format[mnode->type],
 			  indent_level, "");
 
+  if (mnode->type != NODE_TYPE_ROOT)
+    {
+      if (mnode->name)
+        g_string_append_printf (buffer, " name=\"%s\"", mnode->name);
+
+      if (mnode->action_name)
+        g_string_append_printf (buffer, " action=\"%s\"",
+                                g_quark_to_string (mnode->action_name));
+    }
+
+  g_string_append (buffer,
+                   close_tag_format[mnode->type] ? ">\n" : "/>\n");
+
+  for (child = node->children; child != NULL; child = child->next)
+    print_node (self, child, indent_level + 2, buffer);
+
+  if (close_tag_format[mnode->type])
+    g_string_append_printf (buffer, close_tag_format[mnode->type],
+                            indent_level, "");
 }
 
 /**
